from typing import TYPE_CHECKING

from sedaro_base_client.apis.tags import jobs_api
from sedaro_base_client.api_client import ApiResponse

if TYPE_CHECKING:
    from .sedaro_api_client import SedaroApiClient


class SimClient:
    """A client to interact with the Sedaro API simulation (jobs) routes"""
    branch_id: int
    _base_jobs_api_client: jobs_api.JobsApi

    def __init__(self, sedaro_api_client: 'SedaroApiClient', branch_id: int):
        """Instantiate a Sedaro SimClient

        Args:
            sedaro_api_client (`SedaroApiClient`): the `SedaroApiClient`
            branch_id (`int`): id of the desired Sedaro Scenario Branch to interact with its simulations (jobs).
        """
        self._base_jobs_api_client = jobs_api.JobsApi(sedaro_api_client)
        self.branch_Id = branch_id

    def start(self) -> ApiResponse:
        """Starts simulation corresponding to the Sedaro Scenario Branch id that this `SimClient` was instantiated with.

        Returns:
            ApiResponse: response from the start simulation (job) request
        """
        return self._base_jobs_api_client.start_simulation(
            path_params={'branchId': self.branch_Id}
        )

    def get_latest(self) -> ApiResponse:
        """Gets the latest running simulation (job) corresponding to the Sedaro Scenario Branch id that this `SimClient`
        was instantiated with.

        Returns:
            ApiResponse: response from the get simulation (job) request
        """
        return self._base_jobs_api_client.get_simulations(
            path_params={'branchId': self.branch_Id},
            query_params={'latest': ''}
        )

    def terminate(self, job_id: int) -> ApiResponse:
        """Terminate simulation corresponding to the Sedaro Scenario Branch id that this `SimClient` was instantiated
        with and the passed in `job_id`.

        Note: the `job_id` of the "latest" running simulation can be retrieved via:

        ```py
            res = sim_client.get_latest()
            job_id = res.body[0]['id']
            sim_client.terminate(job_id)
        ```

        Args:
            job_id (`int`): id of the simulation (job) to termiante.

        Returns:
            ApiResponse: response from the termiante simulation (job) request
        """
        return self._base_jobs_api_client.terminate_simulation(
            path_params={
                'branchId': self.branch_Id,
                'jobId': job_id
            }
        )



import typing_extensions

from sedaro_base_client.paths import PathValues
from sedaro_base_client.apis.paths.models_branches_branch_id_system_satellite_block_id import ModelsBranchesBranchIdSystemSatelliteBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_body_frame_vectors_ import ModelsBranchesBranchIdSystemGeometryBodyFrameVectors
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_body_frame_vectors_block_id import ModelsBranchesBranchIdSystemGeometryBodyFrameVectorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_surfaces_ import ModelsBranchesBranchIdSystemGeometrySurfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_surfaces_block_id import ModelsBranchesBranchIdSystemGeometrySurfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_surfaces_materials_ import ModelsBranchesBranchIdSystemGeometrySurfacesMaterials
from sedaro_base_client.apis.paths.models_branches_branch_id_system_geometry_surfaces_materials_block_id import ModelsBranchesBranchIdSystemGeometrySurfacesMaterialsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_subsystems_ import ModelsBranchesBranchIdSystemSubsystems
from sedaro_base_client.apis.paths.models_branches_branch_id_system_subsystems_block_id import ModelsBranchesBranchIdSystemSubsystemsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_subsystems_components_ import ModelsBranchesBranchIdSystemSubsystemsComponents
from sedaro_base_client.apis.paths.models_branches_branch_id_system_subsystems_components_block_id import ModelsBranchesBranchIdSystemSubsystemsComponentsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_loads_ import ModelsBranchesBranchIdSystemLoads
from sedaro_base_client.apis.paths.models_branches_branch_id_system_loads_block_id import ModelsBranchesBranchIdSystemLoadsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_system_loads_states_ import ModelsBranchesBranchIdSystemLoadsStates
from sedaro_base_client.apis.paths.models_branches_branch_id_system_loads_states_block_id import ModelsBranchesBranchIdSystemLoadsStatesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_operational_modes_ import ModelsBranchesBranchIdCdhConopsOperationalModes
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_operational_modes_block_id import ModelsBranchesBranchIdCdhConopsOperationalModesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_conditions_ import ModelsBranchesBranchIdCdhConopsConditions
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_conditions_block_id import ModelsBranchesBranchIdCdhConopsConditionsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_group_conditions_ import ModelsBranchesBranchIdCdhConopsGroupConditions
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_group_conditions_block_id import ModelsBranchesBranchIdCdhConopsGroupConditionsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_space_targets_ import ModelsBranchesBranchIdCdhConopsSpaceTargets
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_space_targets_block_id import ModelsBranchesBranchIdCdhConopsSpaceTargetsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_celestial_targets_ import ModelsBranchesBranchIdCdhConopsCelestialTargets
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_celestial_targets_block_id import ModelsBranchesBranchIdCdhConopsCelestialTargetsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_ground_targets_ import ModelsBranchesBranchIdCdhConopsGroundTargets
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_ground_targets_block_id import ModelsBranchesBranchIdCdhConopsGroundTargetsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_target_groups_ import ModelsBranchesBranchIdCdhConopsTargetGroups
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_conops_target_groups_block_id import ModelsBranchesBranchIdCdhConopsTargetGroupsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_types_ import ModelsBranchesBranchIdCdhDataDataTypes
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_types_block_id import ModelsBranchesBranchIdCdhDataDataTypesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_internal_data_interfaces_ import ModelsBranchesBranchIdCdhDataInternalDataInterfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_internal_data_interfaces_block_id import ModelsBranchesBranchIdCdhDataInternalDataInterfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_passive_transmit_interfaces_ import ModelsBranchesBranchIdCdhDataPassiveTransmitInterfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_passive_transmit_interfaces_block_id import ModelsBranchesBranchIdCdhDataPassiveTransmitInterfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_cooperative_transmit_interfaces_ import ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_cooperative_transmit_interfaces_block_id import ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_receive_interfaces_ import ModelsBranchesBranchIdCdhDataReceiveInterfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_receive_interfaces_block_id import ModelsBranchesBranchIdCdhDataReceiveInterfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_busses_ import ModelsBranchesBranchIdCdhDataDataBusses
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_busses_block_id import ModelsBranchesBranchIdCdhDataDataBussesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_storage_ import ModelsBranchesBranchIdCdhDataDataStorage
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_storage_block_id import ModelsBranchesBranchIdCdhDataDataStorageBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_modes_ import ModelsBranchesBranchIdCdhDataDataModes
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_data_modes_block_id import ModelsBranchesBranchIdCdhDataDataModesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_modem_ import ModelsBranchesBranchIdCdhDataModem
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_modem_block_id import ModelsBranchesBranchIdCdhDataModemBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_antenna_ import ModelsBranchesBranchIdCdhDataAntenna
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_antenna_block_id import ModelsBranchesBranchIdCdhDataAntennaBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_laser_comm_module_ import ModelsBranchesBranchIdCdhDataLaserCommModule
from sedaro_base_client.apis.paths.models_branches_branch_id_cdh_data_laser_comm_module_block_id import ModelsBranchesBranchIdCdhDataLaserCommModuleBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_ import ModelsBranchesBranchIdPowerSolarArrays
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_block_id import ModelsBranchesBranchIdPowerSolarArraysBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_panels_ import ModelsBranchesBranchIdPowerSolarArraysPanels
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_panels_block_id import ModelsBranchesBranchIdPowerSolarArraysPanelsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_cells_ import ModelsBranchesBranchIdPowerSolarArraysCells
from sedaro_base_client.apis.paths.models_branches_branch_id_power_solar_arrays_cells_block_id import ModelsBranchesBranchIdPowerSolarArraysCellsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_batteries_block_id import ModelsBranchesBranchIdPowerBatteriesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_batteries_packs_ import ModelsBranchesBranchIdPowerBatteriesPacks
from sedaro_base_client.apis.paths.models_branches_branch_id_power_batteries_packs_block_id import ModelsBranchesBranchIdPowerBatteriesPacksBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_batteries_cells_ import ModelsBranchesBranchIdPowerBatteriesCells
from sedaro_base_client.apis.paths.models_branches_branch_id_power_batteries_cells_block_id import ModelsBranchesBranchIdPowerBatteriesCellsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_eps_power_processor_block_id import ModelsBranchesBranchIdPowerEpsPowerProcessorBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_power_eps_bus_regulators_ import ModelsBranchesBranchIdPowerEpsBusRegulators
from sedaro_base_client.apis.paths.models_branches_branch_id_power_eps_bus_regulators_block_id import ModelsBranchesBranchIdPowerEpsBusRegulatorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_heaters_ import ModelsBranchesBranchIdThermalTempControllersHeaters
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_heaters_block_id import ModelsBranchesBranchIdThermalTempControllersHeatersBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_coolers_ import ModelsBranchesBranchIdThermalTempControllersCoolers
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_coolers_block_id import ModelsBranchesBranchIdThermalTempControllersCoolersBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_states_ import ModelsBranchesBranchIdThermalTempControllersStates
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_temp_controllers_states_block_id import ModelsBranchesBranchIdThermalTempControllersStatesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_thermal_interfaces_ import ModelsBranchesBranchIdThermalThermalInterfaces
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_thermal_interfaces_block_id import ModelsBranchesBranchIdThermalThermalInterfacesBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_thermal_interface_materials_ import ModelsBranchesBranchIdThermalThermalInterfaceMaterials
from sedaro_base_client.apis.paths.models_branches_branch_id_thermal_thermal_interface_materials_block_id import ModelsBranchesBranchIdThermalThermalInterfaceMaterialsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_passive_ import ModelsBranchesBranchIdGncPointingModesPassive
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_passive_block_id import ModelsBranchesBranchIdGncPointingModesPassiveBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_lock_ import ModelsBranchesBranchIdGncPointingModesLock
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_lock_block_id import ModelsBranchesBranchIdGncPointingModesLockBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_max_secondary_align_ import ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlign
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_pointing_modes_max_secondary_align_block_id import ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlignBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_local_vectors_ import ModelsBranchesBranchIdGncReferenceVectorsLocalVectors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_local_vectors_block_id import ModelsBranchesBranchIdGncReferenceVectorsLocalVectorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_celestial_vectors_ import ModelsBranchesBranchIdGncReferenceVectorsCelestialVectors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_celestial_vectors_block_id import ModelsBranchesBranchIdGncReferenceVectorsCelestialVectorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_target_vectors_ import ModelsBranchesBranchIdGncReferenceVectorsTargetVectors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_target_vectors_block_id import ModelsBranchesBranchIdGncReferenceVectorsTargetVectorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_target_group_vectors_ import ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_reference_vectors_target_group_vectors_block_id import ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_direction_sensors_ import ModelsBranchesBranchIdGncSensorsDirectionSensors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_direction_sensors_block_id import ModelsBranchesBranchIdGncSensorsDirectionSensorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_vector_sensors_ import ModelsBranchesBranchIdGncSensorsVectorSensors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_vector_sensors_block_id import ModelsBranchesBranchIdGncSensorsVectorSensorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_optical_attitude_sensors_ import ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_optical_attitude_sensors_block_id import ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_position_sensors_ import ModelsBranchesBranchIdGncSensorsPositionSensors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_position_sensors_block_id import ModelsBranchesBranchIdGncSensorsPositionSensorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_angular_velocity_sensors_ import ModelsBranchesBranchIdGncSensorsAngularVelocitySensors
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_angular_velocity_sensors_block_id import ModelsBranchesBranchIdGncSensorsAngularVelocitySensorsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_circ_fields_of_view_ import ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfView
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_circ_fields_of_view_block_id import ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfViewBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_rect_fields_of_view_ import ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfView
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_rect_fields_of_view_block_id import ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfViewBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_constraints_ import ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraints
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_sensors_fields_of_view_constraints_block_id import ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraintsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_magnetorquers_ import ModelsBranchesBranchIdGncActuatorsMagnetorquers
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_magnetorquers_block_id import ModelsBranchesBranchIdGncActuatorsMagnetorquersBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_reaction_wheels_ import ModelsBranchesBranchIdGncActuatorsReactionWheels
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_reaction_wheels_block_id import ModelsBranchesBranchIdGncActuatorsReactionWheelsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_thrusters_ import ModelsBranchesBranchIdGncActuatorsThrusters
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_actuators_thrusters_block_id import ModelsBranchesBranchIdGncActuatorsThrustersBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_triad_ import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriad
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_triad_block_id import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriadBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_averaging_ import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveraging
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_averaging_block_id import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveragingBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_mekf_ import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekf
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_mekf_block_id import ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekfBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_orbit_determination_ekf_ import ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkf
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_orbit_determination_ekf_block_id import ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkfBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_orbit_determination_gps_ import ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGps
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_orbit_determination_gps_block_id import ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGpsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_control_sliding_mode_ import ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingMode
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_control_sliding_mode_block_id import ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingModeBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_control_pid_ import ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPid
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_attitude_control_pid_block_id import ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPidBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_thrust_control_static_thrust_control_ import ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControl
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_algorithms_thrust_control_static_thrust_control_block_id import ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControlBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_ import ModelsBranchesBranchIdGncFuelReservoirs
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_block_id import ModelsBranchesBranchIdGncFuelReservoirsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_tanks_spherical_ import ModelsBranchesBranchIdGncFuelReservoirsTanksSpherical
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_tanks_spherical_block_id import ModelsBranchesBranchIdGncFuelReservoirsTanksSphericalBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_tanks_spherocylinder_ import ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinder
from sedaro_base_client.apis.paths.models_branches_branch_id_gnc_fuel_reservoirs_tanks_spherocylinder_block_id import ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinderBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_agents_ import ModelsBranchesBranchIdAgents
from sedaro_base_client.apis.paths.models_branches_branch_id_agents_block_id import ModelsBranchesBranchIdAgentsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_clock_configs_ import ModelsBranchesBranchIdClockConfigs
from sedaro_base_client.apis.paths.models_branches_branch_id_clock_configs_block_id import ModelsBranchesBranchIdClockConfigsBlockId
from sedaro_base_client.apis.paths.models_branches_branch_id_orbits_ import ModelsBranchesBranchIdOrbits
from sedaro_base_client.apis.paths.models_branches_branch_id_orbits_block_id import ModelsBranchesBranchIdOrbitsBlockId
from sedaro_base_client.apis.paths.simulations_branches_branch_id_control_ import SimulationsBranchesBranchIdControl
from sedaro_base_client.apis.paths.simulations_branches_branch_id_control_job_id import SimulationsBranchesBranchIdControlJobId
from sedaro_base_client.apis.paths.data_ import Data
from sedaro_base_client.apis.paths.models_branches_branch_id import ModelsBranchesBranchId
from sedaro_base_client.apis.paths.models_branches_branch_idshare_auth_ import ModelsBranchesBranchIdshareAuth
from sedaro_base_client.apis.paths.models_branches_branch_idcommits_ import ModelsBranchesBranchIdcommits
from sedaro_base_client.apis.paths.models_branches_current_branch_id_merge_incoming_branch_id import ModelsBranchesCurrentBranchIdMergeIncomingBranchId
from sedaro_base_client.apis.paths.models_branches_branch_idcommitted_ import ModelsBranchesBranchIdcommitted
from sedaro_base_client.apis.paths.models_branches_branch_idsaved_ import ModelsBranchesBranchIdsaved

PathToApi = typing_extensions.TypedDict(
    'PathToApi',
    {
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SATELLITE_BLOCK_ID: ModelsBranchesBranchIdSystemSatelliteBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_: ModelsBranchesBranchIdSystemGeometryBodyFrameVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_BLOCK_ID: ModelsBranchesBranchIdSystemGeometryBodyFrameVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_: ModelsBranchesBranchIdSystemGeometrySurfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_BLOCK_ID: ModelsBranchesBranchIdSystemGeometrySurfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_: ModelsBranchesBranchIdSystemGeometrySurfacesMaterials,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_BLOCK_ID: ModelsBranchesBranchIdSystemGeometrySurfacesMaterialsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_: ModelsBranchesBranchIdSystemSubsystems,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_BLOCK_ID: ModelsBranchesBranchIdSystemSubsystemsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_: ModelsBranchesBranchIdSystemSubsystemsComponents,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_BLOCK_ID: ModelsBranchesBranchIdSystemSubsystemsComponentsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_: ModelsBranchesBranchIdSystemLoads,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_BLOCK_ID: ModelsBranchesBranchIdSystemLoadsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_: ModelsBranchesBranchIdSystemLoadsStates,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_BLOCK_ID: ModelsBranchesBranchIdSystemLoadsStatesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_: ModelsBranchesBranchIdCdhConopsOperationalModes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_BLOCK_ID: ModelsBranchesBranchIdCdhConopsOperationalModesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_: ModelsBranchesBranchIdCdhConopsConditions,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsConditionsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_: ModelsBranchesBranchIdCdhConopsGroupConditions,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsGroupConditionsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_: ModelsBranchesBranchIdCdhConopsSpaceTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsSpaceTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_: ModelsBranchesBranchIdCdhConopsCelestialTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsCelestialTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_: ModelsBranchesBranchIdCdhConopsGroundTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsGroundTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_: ModelsBranchesBranchIdCdhConopsTargetGroups,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsTargetGroupsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_: ModelsBranchesBranchIdCdhDataDataTypes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataTypesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_: ModelsBranchesBranchIdCdhDataInternalDataInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataInternalDataInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_: ModelsBranchesBranchIdCdhDataPassiveTransmitInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataPassiveTransmitInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_: ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_: ModelsBranchesBranchIdCdhDataReceiveInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataReceiveInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_: ModelsBranchesBranchIdCdhDataDataBusses,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataBussesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_: ModelsBranchesBranchIdCdhDataDataStorage,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataStorageBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_: ModelsBranchesBranchIdCdhDataDataModes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataModesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_: ModelsBranchesBranchIdCdhDataModem,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_BLOCK_ID: ModelsBranchesBranchIdCdhDataModemBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_: ModelsBranchesBranchIdCdhDataAntenna,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_BLOCK_ID: ModelsBranchesBranchIdCdhDataAntennaBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_: ModelsBranchesBranchIdCdhDataLaserCommModule,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_BLOCK_ID: ModelsBranchesBranchIdCdhDataLaserCommModuleBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_: ModelsBranchesBranchIdPowerSolarArrays,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_: ModelsBranchesBranchIdPowerSolarArraysPanels,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysPanelsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_: ModelsBranchesBranchIdPowerSolarArraysCells,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysCellsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_: ModelsBranchesBranchIdPowerBatteriesPacks,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesPacksBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_: ModelsBranchesBranchIdPowerBatteriesCells,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesCellsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_POWERPROCESSOR_BLOCK_ID: ModelsBranchesBranchIdPowerEpsPowerProcessorBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_: ModelsBranchesBranchIdPowerEpsBusRegulators,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_BLOCK_ID: ModelsBranchesBranchIdPowerEpsBusRegulatorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_: ModelsBranchesBranchIdThermalTempControllersHeaters,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersHeatersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_: ModelsBranchesBranchIdThermalTempControllersCoolers,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersCoolersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_: ModelsBranchesBranchIdThermalTempControllersStates,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersStatesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_: ModelsBranchesBranchIdThermalThermalInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_BLOCK_ID: ModelsBranchesBranchIdThermalThermalInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_: ModelsBranchesBranchIdThermalThermalInterfaceMaterials,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_BLOCK_ID: ModelsBranchesBranchIdThermalThermalInterfaceMaterialsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_: ModelsBranchesBranchIdGncPointingModesPassive,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesPassiveBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_: ModelsBranchesBranchIdGncPointingModesLock,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesLockBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_: ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlign,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlignBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsLocalVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsLocalVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsCelestialVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsCelestialVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsTargetVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsTargetVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_: ModelsBranchesBranchIdGncSensorsDirectionSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsDirectionSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_: ModelsBranchesBranchIdGncSensorsVectorSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsVectorSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_: ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_: ModelsBranchesBranchIdGncSensorsPositionSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsPositionSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_: ModelsBranchesBranchIdGncSensorsAngularVelocitySensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsAngularVelocitySensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_: ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfView,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfViewBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_: ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfView,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfViewBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_: ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraints,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraintsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_: ModelsBranchesBranchIdGncActuatorsMagnetorquers,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsMagnetorquersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_: ModelsBranchesBranchIdGncActuatorsReactionWheels,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsReactionWheelsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_: ModelsBranchesBranchIdGncActuatorsThrusters,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsThrustersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriad,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriadBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveraging,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveragingBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekf,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekfBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkf,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkfBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGps,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGpsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingMode,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingModeBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPid,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPidBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_: ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControl,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControlBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_: ModelsBranchesBranchIdGncFuelReservoirs,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherical,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsTanksSphericalBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinder,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinderBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_AGENTS_: ModelsBranchesBranchIdAgents,
        PathValues.MODELS_BRANCHES_BRANCH_ID_AGENTS_BLOCK_ID: ModelsBranchesBranchIdAgentsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_: ModelsBranchesBranchIdClockConfigs,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_BLOCK_ID: ModelsBranchesBranchIdClockConfigsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_ORBITS_: ModelsBranchesBranchIdOrbits,
        PathValues.MODELS_BRANCHES_BRANCH_ID_ORBITS_BLOCK_ID: ModelsBranchesBranchIdOrbitsBlockId,
        PathValues.SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_: SimulationsBranchesBranchIdControl,
        PathValues.SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_JOB_ID: SimulationsBranchesBranchIdControlJobId,
        PathValues.DATA_: Data,
        PathValues.MODELS_BRANCHES_BRANCH_ID: ModelsBranchesBranchId,
        PathValues.MODELS_BRANCHES_BRANCH_IDSHAREAUTH_: ModelsBranchesBranchIdshareAuth,
        PathValues.MODELS_BRANCHES_BRANCH_IDCOMMITS_: ModelsBranchesBranchIdcommits,
        PathValues.MODELS_BRANCHES_CURRENT_BRANCH_ID_MERGE_INCOMING_BRANCH_ID: ModelsBranchesCurrentBranchIdMergeIncomingBranchId,
        PathValues.MODELS_BRANCHES_BRANCH_IDCOMMITTED_: ModelsBranchesBranchIdcommitted,
        PathValues.MODELS_BRANCHES_BRANCH_IDSAVED_: ModelsBranchesBranchIdsaved,
    }
)

path_to_api = PathToApi(
    {
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SATELLITE_BLOCK_ID: ModelsBranchesBranchIdSystemSatelliteBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_: ModelsBranchesBranchIdSystemGeometryBodyFrameVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_BLOCK_ID: ModelsBranchesBranchIdSystemGeometryBodyFrameVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_: ModelsBranchesBranchIdSystemGeometrySurfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_BLOCK_ID: ModelsBranchesBranchIdSystemGeometrySurfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_: ModelsBranchesBranchIdSystemGeometrySurfacesMaterials,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_BLOCK_ID: ModelsBranchesBranchIdSystemGeometrySurfacesMaterialsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_: ModelsBranchesBranchIdSystemSubsystems,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_BLOCK_ID: ModelsBranchesBranchIdSystemSubsystemsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_: ModelsBranchesBranchIdSystemSubsystemsComponents,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_BLOCK_ID: ModelsBranchesBranchIdSystemSubsystemsComponentsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_: ModelsBranchesBranchIdSystemLoads,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_BLOCK_ID: ModelsBranchesBranchIdSystemLoadsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_: ModelsBranchesBranchIdSystemLoadsStates,
        PathValues.MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_BLOCK_ID: ModelsBranchesBranchIdSystemLoadsStatesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_: ModelsBranchesBranchIdCdhConopsOperationalModes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_BLOCK_ID: ModelsBranchesBranchIdCdhConopsOperationalModesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_: ModelsBranchesBranchIdCdhConopsConditions,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsConditionsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_: ModelsBranchesBranchIdCdhConopsGroupConditions,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsGroupConditionsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_: ModelsBranchesBranchIdCdhConopsSpaceTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsSpaceTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_: ModelsBranchesBranchIdCdhConopsCelestialTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsCelestialTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_: ModelsBranchesBranchIdCdhConopsGroundTargets,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsGroundTargetsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_: ModelsBranchesBranchIdCdhConopsTargetGroups,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_BLOCK_ID: ModelsBranchesBranchIdCdhConopsTargetGroupsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_: ModelsBranchesBranchIdCdhDataDataTypes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataTypesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_: ModelsBranchesBranchIdCdhDataInternalDataInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataInternalDataInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_: ModelsBranchesBranchIdCdhDataPassiveTransmitInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataPassiveTransmitInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_: ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataCooperativeTransmitInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_: ModelsBranchesBranchIdCdhDataReceiveInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_BLOCK_ID: ModelsBranchesBranchIdCdhDataReceiveInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_: ModelsBranchesBranchIdCdhDataDataBusses,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataBussesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_: ModelsBranchesBranchIdCdhDataDataStorage,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataStorageBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_: ModelsBranchesBranchIdCdhDataDataModes,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_BLOCK_ID: ModelsBranchesBranchIdCdhDataDataModesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_: ModelsBranchesBranchIdCdhDataModem,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_BLOCK_ID: ModelsBranchesBranchIdCdhDataModemBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_: ModelsBranchesBranchIdCdhDataAntenna,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_BLOCK_ID: ModelsBranchesBranchIdCdhDataAntennaBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_: ModelsBranchesBranchIdCdhDataLaserCommModule,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_BLOCK_ID: ModelsBranchesBranchIdCdhDataLaserCommModuleBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_: ModelsBranchesBranchIdPowerSolarArrays,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_: ModelsBranchesBranchIdPowerSolarArraysPanels,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysPanelsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_: ModelsBranchesBranchIdPowerSolarArraysCells,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_BLOCK_ID: ModelsBranchesBranchIdPowerSolarArraysCellsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_: ModelsBranchesBranchIdPowerBatteriesPacks,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesPacksBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_: ModelsBranchesBranchIdPowerBatteriesCells,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_BLOCK_ID: ModelsBranchesBranchIdPowerBatteriesCellsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_POWERPROCESSOR_BLOCK_ID: ModelsBranchesBranchIdPowerEpsPowerProcessorBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_: ModelsBranchesBranchIdPowerEpsBusRegulators,
        PathValues.MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_BLOCK_ID: ModelsBranchesBranchIdPowerEpsBusRegulatorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_: ModelsBranchesBranchIdThermalTempControllersHeaters,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersHeatersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_: ModelsBranchesBranchIdThermalTempControllersCoolers,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersCoolersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_: ModelsBranchesBranchIdThermalTempControllersStates,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_BLOCK_ID: ModelsBranchesBranchIdThermalTempControllersStatesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_: ModelsBranchesBranchIdThermalThermalInterfaces,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_BLOCK_ID: ModelsBranchesBranchIdThermalThermalInterfacesBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_: ModelsBranchesBranchIdThermalThermalInterfaceMaterials,
        PathValues.MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_BLOCK_ID: ModelsBranchesBranchIdThermalThermalInterfaceMaterialsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_: ModelsBranchesBranchIdGncPointingModesPassive,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesPassiveBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_: ModelsBranchesBranchIdGncPointingModesLock,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesLockBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_: ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlign,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_BLOCK_ID: ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlignBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsLocalVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsLocalVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsCelestialVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsCelestialVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsTargetVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsTargetVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_: ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_BLOCK_ID: ModelsBranchesBranchIdGncReferenceVectorsTargetGroupVectorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_: ModelsBranchesBranchIdGncSensorsDirectionSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsDirectionSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_: ModelsBranchesBranchIdGncSensorsVectorSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsVectorSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_: ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_: ModelsBranchesBranchIdGncSensorsPositionSensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsPositionSensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_: ModelsBranchesBranchIdGncSensorsAngularVelocitySensors,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsAngularVelocitySensorsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_: ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfView,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewCircFieldsOfViewBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_: ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfView,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfViewBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_: ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraints,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_BLOCK_ID: ModelsBranchesBranchIdGncSensorsFieldsOfViewConstraintsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_: ModelsBranchesBranchIdGncActuatorsMagnetorquers,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsMagnetorquersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_: ModelsBranchesBranchIdGncActuatorsReactionWheels,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsReactionWheelsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_: ModelsBranchesBranchIdGncActuatorsThrusters,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_BLOCK_ID: ModelsBranchesBranchIdGncActuatorsThrustersBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriad,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriadBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveraging,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationAveragingBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekf,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationMekfBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkf,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationEkfBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGps,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGpsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingMode,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlSlidingModeBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPid,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsAttitudeControlPidBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_: ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControl,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_BLOCK_ID: ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControlBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_: ModelsBranchesBranchIdGncFuelReservoirs,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherical,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsTanksSphericalBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinder,
        PathValues.MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_BLOCK_ID: ModelsBranchesBranchIdGncFuelReservoirsTanksSpherocylinderBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_AGENTS_: ModelsBranchesBranchIdAgents,
        PathValues.MODELS_BRANCHES_BRANCH_ID_AGENTS_BLOCK_ID: ModelsBranchesBranchIdAgentsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_: ModelsBranchesBranchIdClockConfigs,
        PathValues.MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_BLOCK_ID: ModelsBranchesBranchIdClockConfigsBlockId,
        PathValues.MODELS_BRANCHES_BRANCH_ID_ORBITS_: ModelsBranchesBranchIdOrbits,
        PathValues.MODELS_BRANCHES_BRANCH_ID_ORBITS_BLOCK_ID: ModelsBranchesBranchIdOrbitsBlockId,
        PathValues.SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_: SimulationsBranchesBranchIdControl,
        PathValues.SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_JOB_ID: SimulationsBranchesBranchIdControlJobId,
        PathValues.DATA_: Data,
        PathValues.MODELS_BRANCHES_BRANCH_ID: ModelsBranchesBranchId,
        PathValues.MODELS_BRANCHES_BRANCH_IDSHAREAUTH_: ModelsBranchesBranchIdshareAuth,
        PathValues.MODELS_BRANCHES_BRANCH_IDCOMMITS_: ModelsBranchesBranchIdcommits,
        PathValues.MODELS_BRANCHES_CURRENT_BRANCH_ID_MERGE_INCOMING_BRANCH_ID: ModelsBranchesCurrentBranchIdMergeIncomingBranchId,
        PathValues.MODELS_BRANCHES_BRANCH_IDCOMMITTED_: ModelsBranchesBranchIdcommitted,
        PathValues.MODELS_BRANCHES_BRANCH_IDSAVED_: ModelsBranchesBranchIdsaved,
    }
)



from sedaro_base_client.paths.models_branches_branch_id_gnc_algorithms_orbit_determination_gps_.post import ApiForpost


class ModelsBranchesBranchIdGncAlgorithmsOrbitDeterminationGps(
    ApiForpost,
):
    pass






ENGINE_MAP = {
    '0': 'gnc',
    '1': 'cdh',
    '2': 'power',
    '3': 'thermal',
}
ENGINE_EXPANSION = {
    'gnc': 'Guidance, Navigation, & Control',
    'cdh': 'Command & Data Handling',
    'power': 'Power',
    'thermal': 'Thermal',
}
STATUS_ICON_MAP = {
    "SUCCEEDED": "✅",
    "FAILED": "❌",
    "TERMINATED": "❌",
    "PAUSED": "⏸️",
    "PENDING": "⌛",
    "RUNNING": "⌛",
}
HFILL = 75


def hfill(char="-", len=HFILL):
    print(char * len)


def progress_bar(progress):
    blocks = int(progress * 50 / 100)
    bar = '[' + ('■' * blocks + '□'*(50 - blocks)).ljust(50) + f'] ({progress:.2f}%)'
    print(bar, end='\r')


def _element_id_dict(agent_data):
    '''Break out all blocks into a dict where each key is an ID.'''
    out = {}
    for entry in agent_data.values():
        if isinstance(entry, dict):
            for id_, value in entry.items():
                if 'id' in value:
                    if id_ in out:
                        raise ValueError(f"Duplicate ID {id_}")
                    else:
                        out[id_] = value

    return out


def _get_agent_id_name_map(meta):
    '''Get mapping from agent ID to name.'''
    return {id_: entry['name'] for id_, entry in meta['structure']['scenario']['Agent'].items()}

def _simplify_series(engine_data: dict, blocks: dict) -> dict:
    '''Build a simplified series data structure

    Creates a dictionary with the following hierarchy:
        Block ID (or root)
            Variable Name
    '''
    data = {'root': {}}
    for key, value in engine_data.items():
        if key in blocks:
            data[key] = {}
            for subkey, subvalue in value.items():
                data[key][subkey] = subvalue
        elif "/" in key:
            # Ignore engine variables
            continue
        else:
            data['root'][key] = value
    return data


def _restructure_data(series, agents, meta, agent_map):
    '''Build a simplified internal data structure.

    Creates a dictionary with the following key hierarchy:

        Agent Name
            Engine Name (gnc, cdh, power, thermal)
                Time
                Series
                    Block ID (or root)
                        Variable Name
    '''
    data = {}
    blocks = {}
    for series_key in series:
        agent_id, engine_id = series_key.split("/")
        agent_simbed_id = agent_map[agent_id]
        agent_name = agents[agent_simbed_id]
        engine_name = ENGINE_MAP[engine_id]

        if agent_name not in data:
            data[agent_name] = {}

        time, sub_series = series[series_key]
        if agent_simbed_id not in blocks:
            blocks[agent_simbed_id] = _element_id_dict(meta['structure']['agents'].get(agent_id, {}))
        data[agent_name][engine_name] = {
            'time': time,
            'series': _simplify_series(sub_series[agent_id], blocks[agent_simbed_id])
        }
    return data, blocks


def _get_series_type(series):
    for entry in series:
        if entry is not None:
            return type(entry).__name__
    else:
        return "None"



from sedaro_base_client.paths.models_branches_branch_id_thermal_temp_controllers_heaters_.post import ApiForpost


class ModelsBranchesBranchIdThermalTempControllersHeaters(
    ApiForpost,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_gnc_actuators_reaction_wheels_block_id.delete import ApiFordelete
from sedaro_base_client.paths.models_branches_branch_id_gnc_actuators_reaction_wheels_block_id.patch import ApiForpatch


class ModelsBranchesBranchIdGncActuatorsReactionWheelsBlockId(
    ApiFordelete,
    ApiForpatch,
):
    pass



from importlib import import_module
from dataclasses import dataclass
from sedaro_base_client.api_client import Api
from typing import TYPE_CHECKING, Tuple, Union, List
from typing_extensions import Literal
from pydash.strings import snake_case
# from functools import cached_property # doesn't work python 3.7

from .settings import CREATE, UPDATE, BASE_PACKAGE_NAME
from .block_client import BlockClient
from .utils import get_snake_and_pascal_case, sanitize_and_enforce_id_in_branch, get_class_from_module, temp_crud
from .exceptions import NoBlockFoundError

if TYPE_CHECKING:
    from sedaro_base_client import schemas
    from .sedaro_api_client import SedaroApiClient
    from .branch_client import BranchClient


@dataclass
class BlockClassClient:
    '''Class for getting `BlockClient`s associated with Sedaro Blocks of this class type'''
    _block_name: str
    '''Name of the Sedaro Block class this `BlockClassClient` is set up to interact with'''
    _block_openapi_instance: Union[Api, None]
    '''
    The api instance instantiated with the appropriate `SedaroApiClient` to interact with when CRUDing Blocks. This
    property should be `None` for some limited Sedaro `Block` options (such as ConOps) that have no associated API.
    '''
    _branch_client: 'BranchClient'
    '''The `BranchClient` this `BlockClassClient` is connected to'''

    def __str__(self) -> str:
        return f'BlockClassClient({self._block_name}, branch={self._branch_client.id})'

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return hash(f'{self.__class__.__name__}-{self._block_name}-{self._branch_client.id}')

    @property
    def _sedaro_client(self) -> 'SedaroApiClient':
        '''The `SedaroApiClient` this `BlockClassClient` was accessed through'''
        return self._branch_client._sedaro_client

    @property
    def _create_class(self) -> 'schemas.DictSchema':
        '''The model class to instantiate with appropriate kwargs when creating a Sedaro Block'''
        return self._get_create_or_update_block_model(CREATE)

    @property
    def _update_class(self) -> 'schemas.DictSchema':
        '''The model class to instantiate with appropriate kwargs when updating a Sedaro Block'''
        return self._get_create_or_update_block_model(UPDATE)

    # @cached_property
    @property
    def _block_group(self) -> str:
        '''The name of the Sedaro Block Group this type of Sedaro Block is stored in'''
        return self._branch_client._block_class_to_block_group_map[self._block_name]

        # Below can get block group if ever choose to not send blockClassToBlockGroupMap in response
        # PROPERTIES = 'properties'
        # REF = '$ref'
        # ANY_OF = 'anyOf'
        # ALL_OF = 'allOf'

        # block_group_type = None

        # # Traverse branch schema to figure out block group type, then traverse again to get matching block group
        # for k, v in self._branch_client.dataSchema['definitions'].items():
        #     # filter through all block group types
        #     if PROPERTIES in v and all(attr in v[PROPERTIES] for attr in ('name', 'collection', 'data')):
        #         # TODO: `if k.endswith('BG')` <-- evalutes to same as ^^^ if we always follow this convention for naming
        #         # block group classes. Would still like a safer way than either of these options though.
        #         blockClassOrClasses: Dict = v[PROPERTIES]['data']['additionalProperties']

        #         blockTypes = [blockClassOrClasses[REF]] if REF in blockClassOrClasses \
        #             else [v[REF] for v in blockClassOrClasses[ANY_OF]]

        #         if any(bT.endswith(self._block_name) for bT in blockTypes):
        #             block_group_type = k
        #             break

        # # check block group types (`bGT`) of all block groups to find which one matches `block_group_type`
        # for k, v in self._branch_client.dataSchema[PROPERTIES].items():
        #     if ALL_OF in v and any(bGT[REF].endswith(block_group_type) for bGT in v[ALL_OF]):
        #         return k

        # this shouldn't ever happen:
        # raise ValueError(
        #     f'Unable to find a block group containing the block name {self._block_name}')

    def _get_create_or_update_block_model(self, create_or_update: Literal['create', 'update']) -> 'schemas.DictSchema':
        """Gets the model class to used to validate the data to create or update a Sedaro Block

        Args:
            create_or_update (Literal['create', 'update']): the action `'create'` or `'update'`

        Raises:
            ValueError: if you don't pass `'create'` or `'update'` for `create_or_update`

        Returns:
            schemas.DictSchema: the model used for validating
        """
        create_or_update = create_or_update.lower()
        if create_or_update not in [CREATE, UPDATE]:
            raise ValueError(
                "The create_or_update arg must be either a string of either 'create' or 'update'.")

        block_snake, block_pascal = get_snake_and_pascal_case(self._block_name)

        crud_module_path = f'{BASE_PACKAGE_NAME}.model.{block_snake}'

        return get_class_from_module(
            import_module(f'{crud_module_path}_{create_or_update}'),
            target_class=f'{block_pascal}{create_or_update.capitalize()}'
        )

    def create(self, timeout: Union[int, Tuple] = None, **body) -> BlockClient:
        """Creates a Sedaro Block of the given type in the corresponding Branch.

        Args:
            timeout (Union[int, Tuple], optional): the timeout used by the REST client. Defaults to `None`.
            **body (any): all remaining kwargs should correspond with required and optional fields on the Sedaro Block.
            They will form the request `body` dictionary.

        Returns:
            BlockClient: a client to interact with the created Sedaro Block
        """

        try:
            if self._block_openapi_instance is None:
                raise AttributeError
            create_method = temp_crud(self._sedaro_client, 'POST')(getattr(
                self._block_openapi_instance, f'{CREATE}_{snake_case(self._block_name)}'
            ))
        except AttributeError:
            raise AttributeError(
                f'There is no create method on a "{self._block_name}" {self.__class__.__name__} because this type of Sedaro Block is not createable.')

        res = create_method(
            # body=self._create_class(**body), # TODO: temp_crud
            body=body,
            path_params={'branchId': self._branch_client.id},
            timeout=timeout
        )
        block_id = self._branch_client._process_block_crud_response(res)

        return BlockClient(block_id, self)

    def get(self, id: Union[str, int]) -> BlockClient:
        """Gets a `BlockClient` of the desired type associated with the Sedaro Block of the given `id`.

        Args:
            id (Union[str, int]): `id` of the desired Sedaro Block

        Raises:
            TypeError: if the `id` is not an integer or an integer string
            KeyError: if no corresponding Block exists of the desired type

        Returns:
            BlockClient: a client to interact with the corresponding Sedaro Block
        """
        id = sanitize_and_enforce_id_in_branch(self._branch_client, id)

        # in addition to checks in ^^^ also make sure is the correct type for this block class client
        if self._branch_client._block_id_to_type_map[id] != self._block_name:
            raise KeyError(
                f'There is no "{self._block_name}" with id "{id}" in this Branch.')

        return BlockClient(id, self)

    def get_all_ids(self) -> List[str]:
        """Gets a `list` of `id`s corresponding to all Sedaro Blocks of the given type in this Branch. If there are no
        corresponding Blocks, returns an empty `list`.

        Returns:
            List[str]: list of `id`s
        """
        return [
            id for id in self._branch_client.data[self._block_group].keys()
            if self._branch_client._block_id_to_type_map[id] == self._block_name
        ]

    def get_all(self) -> List['BlockClient']:
        """Gets a `list` of all `BlockClient` instances corresponding to all Sedaro Blocks of the given type in this
        Branch. If there are no corresponding Blocks, returns an empty `list`.

        Returns:
            List['BlockClient']: a list of `BlockClient` instances corresponding to Sedaro Blocks in this Branch
        """
        return [BlockClient(id, self) for id in self.get_all_ids()]

    # def get_where(self, **property_values) -> List['BlockClient']:
    #     """TODO: not working yet properly due to properties that have become `DynamicSchema`... comparisons too often
    #     return False when they shouldn't

    #     Gets a filtered `list` of all `BlockClient` instances corresponding to all Sedaro Blocks of the given type in
    #     this Branch. Blocks are filtered by property/values passed as kwargs. If there are no corresponding Blocks,
    #     returns an empty `list`.

    #     **property_values:
    #         any: keys to check for given values on the Sedaro Blocks

    #     Returns:
    #         List['BlockClient']: a filtered list of `BlockClient` instances corresponding to Sedaro Blocks in this
    #         Branch
    #     """
    #     return [
    #         b_c for b_c in self.get_all() if all(getattr(b_c, k) == v for k, v in property_values.items())
    #     ]

    def get_first(self):
        """Returns a `BlockClient` associated with the least recently added (lowest `id`) Sedaro Block of the desired
        type.

        Raises:
            NoBlockFoundError: if no Blocks of the desired type exist in this Branch

        Returns:
            BlockClient: a client to interact with the corresponding Sedaro Block
        """
        all_ids = self.get_all_ids()
        if not len(all_ids):
            raise NoBlockFoundError(
                f'No "{self._block_name}" Blocks exist in this Branch.'
            )
        return BlockClient(sorted(all_ids, key=int)[0], self)

    def get_last(self):
        """Returns a `BlockClient` associated with the most recently added (highest `id`) Sedaro Block of the desired
        type.

        Raises:
            NoBlockFoundError: if no Blocks of the desired type exist in this Branch

        Returns:
            BlockClient: a client to interact with the corresponding Sedaro Block
        """
        all_ids = self.get_all_ids()
        if not len(all_ids):
            raise NoBlockFoundError(
                f'No "{self._block_name}" Blocks exist in this Branch.'
            )
        return BlockClient(sorted(all_ids, key=int)[-1], self)



from sedaro_base_client.paths.models_branches_branch_id_gnc_sensors_optical_attitude_sensors_.post import ApiForpost


class ModelsBranchesBranchIdGncSensorsOpticalAttitudeSensors(
    ApiForpost,
):
    pass



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from sedaro_base_client.apis.path_to_api import path_to_api

import enum


class PathValues(str, enum.Enum):
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_SATELLITE_BLOCK_ID = "/models/branches/{branchId}/system/satellite/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_ = "/models/branches/{branchId}/system/geometry/body-frame-vectors/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_BODYFRAMEVECTORS_BLOCK_ID = "/models/branches/{branchId}/system/geometry/body-frame-vectors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_ = "/models/branches/{branchId}/system/geometry/surfaces/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_BLOCK_ID = "/models/branches/{branchId}/system/geometry/surfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_ = "/models/branches/{branchId}/system/geometry/surfaces/materials/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_GEOMETRY_SURFACES_MATERIALS_BLOCK_ID = "/models/branches/{branchId}/system/geometry/surfaces/materials/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_ = "/models/branches/{branchId}/system/subsystems/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_BLOCK_ID = "/models/branches/{branchId}/system/subsystems/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_ = "/models/branches/{branchId}/system/subsystems/components/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_SUBSYSTEMS_COMPONENTS_BLOCK_ID = "/models/branches/{branchId}/system/subsystems/components/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_ = "/models/branches/{branchId}/system/loads/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_BLOCK_ID = "/models/branches/{branchId}/system/loads/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_ = "/models/branches/{branchId}/system/loads/states/"
    MODELS_BRANCHES_BRANCH_ID_SYSTEM_LOADS_STATES_BLOCK_ID = "/models/branches/{branchId}/system/loads/states/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_ = "/models/branches/{branchId}/cdh/conops/operational-modes/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_OPERATIONALMODES_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/operational-modes/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_ = "/models/branches/{branchId}/cdh/conops/conditions/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CONDITIONS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/conditions/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_ = "/models/branches/{branchId}/cdh/conops/group-conditions/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUPCONDITIONS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/group-conditions/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_ = "/models/branches/{branchId}/cdh/conops/space-targets/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_SPACETARGETS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/space-targets/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_ = "/models/branches/{branchId}/cdh/conops/celestial-targets/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_CELESTIALTARGETS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/celestial-targets/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_ = "/models/branches/{branchId}/cdh/conops/ground-targets/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_GROUNDTARGETS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/ground-targets/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_ = "/models/branches/{branchId}/cdh/conops/target-groups/"
    MODELS_BRANCHES_BRANCH_ID_CDH_CONOPS_TARGETGROUPS_BLOCK_ID = "/models/branches/{branchId}/cdh/conops/target-groups/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_ = "/models/branches/{branchId}/cdh/data/data-types/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATATYPES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/data-types/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_ = "/models/branches/{branchId}/cdh/data/internal-data-interfaces/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_INTERNALDATAINTERFACES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/internal-data-interfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_ = "/models/branches/{branchId}/cdh/data/passive-transmit-interfaces/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_PASSIVETRANSMITINTERFACES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/passive-transmit-interfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_ = "/models/branches/{branchId}/cdh/data/cooperative-transmit-interfaces/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_COOPERATIVETRANSMITINTERFACES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/cooperative-transmit-interfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_ = "/models/branches/{branchId}/cdh/data/receive-interfaces/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_RECEIVEINTERFACES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/receive-interfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_ = "/models/branches/{branchId}/cdh/data/data-busses/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATABUSSES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/data-busses/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_ = "/models/branches/{branchId}/cdh/data/data-storage/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATASTORAGE_BLOCK_ID = "/models/branches/{branchId}/cdh/data/data-storage/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_ = "/models/branches/{branchId}/cdh/data/data-modes/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_DATAMODES_BLOCK_ID = "/models/branches/{branchId}/cdh/data/data-modes/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_ = "/models/branches/{branchId}/cdh/data/modem/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_MODEM_BLOCK_ID = "/models/branches/{branchId}/cdh/data/modem/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_ = "/models/branches/{branchId}/cdh/data/antenna/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_ANTENNA_BLOCK_ID = "/models/branches/{branchId}/cdh/data/antenna/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_ = "/models/branches/{branchId}/cdh/data/laser-comm-module/"
    MODELS_BRANCHES_BRANCH_ID_CDH_DATA_LASERCOMMMODULE_BLOCK_ID = "/models/branches/{branchId}/cdh/data/laser-comm-module/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_ = "/models/branches/{branchId}/power/solar-arrays/"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_BLOCK_ID = "/models/branches/{branchId}/power/solar-arrays/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_ = "/models/branches/{branchId}/power/solar-arrays/panels/"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_PANELS_BLOCK_ID = "/models/branches/{branchId}/power/solar-arrays/panels/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_ = "/models/branches/{branchId}/power/solar-arrays/cells/"
    MODELS_BRANCHES_BRANCH_ID_POWER_SOLARARRAYS_CELLS_BLOCK_ID = "/models/branches/{branchId}/power/solar-arrays/cells/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_BLOCK_ID = "/models/branches/{branchId}/power/batteries/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_ = "/models/branches/{branchId}/power/batteries/packs/"
    MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_PACKS_BLOCK_ID = "/models/branches/{branchId}/power/batteries/packs/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_ = "/models/branches/{branchId}/power/batteries/cells/"
    MODELS_BRANCHES_BRANCH_ID_POWER_BATTERIES_CELLS_BLOCK_ID = "/models/branches/{branchId}/power/batteries/cells/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_EPS_POWERPROCESSOR_BLOCK_ID = "/models/branches/{branchId}/power/eps/power-processor/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_ = "/models/branches/{branchId}/power/eps/bus-regulators/"
    MODELS_BRANCHES_BRANCH_ID_POWER_EPS_BUSREGULATORS_BLOCK_ID = "/models/branches/{branchId}/power/eps/bus-regulators/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_ = "/models/branches/{branchId}/thermal/temp-controllers/heaters/"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_HEATERS_BLOCK_ID = "/models/branches/{branchId}/thermal/temp-controllers/heaters/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_ = "/models/branches/{branchId}/thermal/temp-controllers/coolers/"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_COOLERS_BLOCK_ID = "/models/branches/{branchId}/thermal/temp-controllers/coolers/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_ = "/models/branches/{branchId}/thermal/temp-controllers/states/"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_TEMPCONTROLLERS_STATES_BLOCK_ID = "/models/branches/{branchId}/thermal/temp-controllers/states/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_ = "/models/branches/{branchId}/thermal/thermal-interfaces/"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACES_BLOCK_ID = "/models/branches/{branchId}/thermal/thermal-interfaces/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_ = "/models/branches/{branchId}/thermal/thermal-interface-materials/"
    MODELS_BRANCHES_BRANCH_ID_THERMAL_THERMALINTERFACEMATERIALS_BLOCK_ID = "/models/branches/{branchId}/thermal/thermal-interface-materials/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_ = "/models/branches/{branchId}/gnc/pointing-modes/passive/"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_PASSIVE_BLOCK_ID = "/models/branches/{branchId}/gnc/pointing-modes/passive/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_ = "/models/branches/{branchId}/gnc/pointing-modes/lock/"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_LOCK_BLOCK_ID = "/models/branches/{branchId}/gnc/pointing-modes/lock/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_ = "/models/branches/{branchId}/gnc/pointing-modes/max-secondary-align/"
    MODELS_BRANCHES_BRANCH_ID_GNC_POINTINGMODES_MAXSECONDARYALIGN_BLOCK_ID = "/models/branches/{branchId}/gnc/pointing-modes/max-secondary-align/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_ = "/models/branches/{branchId}/gnc/reference-vectors/local-vectors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_LOCALVECTORS_BLOCK_ID = "/models/branches/{branchId}/gnc/reference-vectors/local-vectors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_ = "/models/branches/{branchId}/gnc/reference-vectors/celestial-vectors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_CELESTIALVECTORS_BLOCK_ID = "/models/branches/{branchId}/gnc/reference-vectors/celestial-vectors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_ = "/models/branches/{branchId}/gnc/reference-vectors/target-vectors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETVECTORS_BLOCK_ID = "/models/branches/{branchId}/gnc/reference-vectors/target-vectors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_ = "/models/branches/{branchId}/gnc/reference-vectors/target-group-vectors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_REFERENCEVECTORS_TARGETGROUPVECTORS_BLOCK_ID = "/models/branches/{branchId}/gnc/reference-vectors/target-group-vectors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_ = "/models/branches/{branchId}/gnc/sensors/direction-sensors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_DIRECTIONSENSORS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/direction-sensors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_ = "/models/branches/{branchId}/gnc/sensors/vector-sensors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_VECTORSENSORS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/vector-sensors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_ = "/models/branches/{branchId}/gnc/sensors/optical-attitude-sensors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_OPTICALATTITUDESENSORS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/optical-attitude-sensors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_ = "/models/branches/{branchId}/gnc/sensors/position-sensors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_POSITIONSENSORS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/position-sensors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_ = "/models/branches/{branchId}/gnc/sensors/angular-velocity-sensors/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_ANGULARVELOCITYSENSORS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/angular-velocity-sensors/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_ = "/models/branches/{branchId}/gnc/sensors/fields-of-view/circ-fields-of-view/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CIRCFIELDSOFVIEW_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/fields-of-view/circ-fields-of-view/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_ = "/models/branches/{branchId}/gnc/sensors/fields-of-view/rect-fields-of-view/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_RECTFIELDSOFVIEW_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/fields-of-view/rect-fields-of-view/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_ = "/models/branches/{branchId}/gnc/sensors/fields-of-view/constraints/"
    MODELS_BRANCHES_BRANCH_ID_GNC_SENSORS_FIELDSOFVIEW_CONSTRAINTS_BLOCK_ID = "/models/branches/{branchId}/gnc/sensors/fields-of-view/constraints/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_ = "/models/branches/{branchId}/gnc/actuators/magnetorquers/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_MAGNETORQUERS_BLOCK_ID = "/models/branches/{branchId}/gnc/actuators/magnetorquers/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_ = "/models/branches/{branchId}/gnc/actuators/reaction-wheels/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_REACTIONWHEELS_BLOCK_ID = "/models/branches/{branchId}/gnc/actuators/reaction-wheels/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_ = "/models/branches/{branchId}/gnc/actuators/thrusters/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ACTUATORS_THRUSTERS_BLOCK_ID = "/models/branches/{branchId}/gnc/actuators/thrusters/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_ = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/triad/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_TRIAD_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/triad/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_ = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/averaging/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_AVERAGING_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/averaging/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_ = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/mekf/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDEDETERMINATION_MEKF_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/attitude-determination/mekf/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_ = "/models/branches/{branchId}/gnc/algorithms/orbit-determination/ekf/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_EKF_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/orbit-determination/ekf/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_ = "/models/branches/{branchId}/gnc/algorithms/orbit-determination/gps/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ORBITDETERMINATION_GPS_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/orbit-determination/gps/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_ = "/models/branches/{branchId}/gnc/algorithms/attitude-control/sliding-mode/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_SLIDINGMODE_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/attitude-control/sliding-mode/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_ = "/models/branches/{branchId}/gnc/algorithms/attitude-control/pid/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_ATTITUDECONTROL_PID_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/attitude-control/pid/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_ = "/models/branches/{branchId}/gnc/algorithms/thrust-control/static-thrust-control/"
    MODELS_BRANCHES_BRANCH_ID_GNC_ALGORITHMS_THRUSTCONTROL_STATICTHRUSTCONTROL_BLOCK_ID = "/models/branches/{branchId}/gnc/algorithms/thrust-control/static-thrust-control/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_ = "/models/branches/{branchId}/gnc/fuel-reservoirs/"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_BLOCK_ID = "/models/branches/{branchId}/gnc/fuel-reservoirs/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_ = "/models/branches/{branchId}/gnc/fuel-reservoirs/tanks/spherical/"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHERICAL_BLOCK_ID = "/models/branches/{branchId}/gnc/fuel-reservoirs/tanks/spherical/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_ = "/models/branches/{branchId}/gnc/fuel-reservoirs/tanks/spherocylinder/"
    MODELS_BRANCHES_BRANCH_ID_GNC_FUELRESERVOIRS_TANKS_SPHEROCYLINDER_BLOCK_ID = "/models/branches/{branchId}/gnc/fuel-reservoirs/tanks/spherocylinder/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_AGENTS_ = "/models/branches/{branchId}/agents/"
    MODELS_BRANCHES_BRANCH_ID_AGENTS_BLOCK_ID = "/models/branches/{branchId}/agents/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_ = "/models/branches/{branchId}/clock-configs/"
    MODELS_BRANCHES_BRANCH_ID_CLOCKCONFIGS_BLOCK_ID = "/models/branches/{branchId}/clock-configs/{blockId}"
    MODELS_BRANCHES_BRANCH_ID_ORBITS_ = "/models/branches/{branchId}/orbits/"
    MODELS_BRANCHES_BRANCH_ID_ORBITS_BLOCK_ID = "/models/branches/{branchId}/orbits/{blockId}"
    SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_ = "/simulations/branches/{branchId}/control/"
    SIMULATIONS_BRANCHES_BRANCH_ID_CONTROL_JOB_ID = "/simulations/branches/{branchId}/control/{jobId}"
    DATA_ = "/data/"
    MODELS_BRANCHES_BRANCH_ID = "/models/branches/{branchId}"
    MODELS_BRANCHES_BRANCH_IDSHAREAUTH_ = "/models/branches/{branchId}share-auth/"
    MODELS_BRANCHES_BRANCH_IDCOMMITS_ = "/models/branches/{branchId}commits/"
    MODELS_BRANCHES_CURRENT_BRANCH_ID_MERGE_INCOMING_BRANCH_ID = "/models/branches/{currentBranchId}/merge/{incomingBranchId}"
    MODELS_BRANCHES_BRANCH_IDCOMMITTED_ = "/models/branches/{branchId}committed/"
    MODELS_BRANCHES_BRANCH_IDSAVED_ = "/models/branches/{branchId}saved/"



from sedaro_base_client.paths.models_branches_branch_id_gnc_sensors_fields_of_view_rect_fields_of_view_block_id.delete import ApiFordelete
from sedaro_base_client.paths.models_branches_branch_id_gnc_sensors_fields_of_view_rect_fields_of_view_block_id.patch import ApiForpatch


class ModelsBranchesBranchIdGncSensorsFieldsOfViewRectFieldsOfViewBlockId(
    ApiFordelete,
    ApiForpatch,
):
    pass



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from sedaro_base_client.apis.tag_to_api import tag_to_api

import enum


class TagValues(str, enum.Enum):
    BRANCHES = "Branches"
    DATA = "Data"
    AGENT = "Agent"
    ANGULAR_VELOCITY_SENSOR = "Angular Velocity Sensor"
    ANTENNA = "Antenna"
    AVERAGING_ALGORITHM = "Averaging Algorithm"
    BATTERY = "Battery"
    BATTERY_CELL = "Battery Cell"
    BATTERY_PACK = "Battery Pack"
    BODY_FRAME_VECTOR = "Body Frame Vector"
    BUS_REGULATOR = "Bus Regulator"
    CELESTIAL_TARGET = "Celestial Target"
    CELESTIAL_VECTOR = "Celestial Vector"
    CIRCULAR_FIELD_OF_VIEW = "Circular Field Of View"
    CLOCK_CONFIG = "Clock Config"
    COMPONENT = "Component"
    CONDITION = "Condition"
    CONSTANT_LOAD = "Constant Load"
    COOLER = "Cooler"
    COOPERATIVE_TRANSMIT_INTERFACE = "Cooperative Transmit Interface"
    DATA_BUS = "Data Bus"
    DATA_MODE = "Data Mode"
    DATA_STORAGE = "Data Storage"
    DATA_TYPE = "Data Type"
    DIRECTION_SENSOR = "Direction Sensor"
    EKF_ALGORITHM = "EKF Algorithm"
    FOV_CONSTRAINT = "FOV Constraint"
    FUEL_RESERVOIR = "Fuel Reservoir"
    GPS_ALGORITHM = "GPS Algorithm"
    GROUND_TARGET = "Ground Target"
    GROUP_CONDITION = "Group Condition"
    HEATER = "Heater"
    INTERNAL_DATA_INTERFACE = "Internal Data Interface"
    JOBS = "Jobs"
    LASER_COMM_MODULE = "Laser Comm Module"
    LOAD_STATE = "Load State"
    LOCAL_VECTOR = "Local Vector"
    LOCK_POINTING_MODE = "Lock Pointing Mode"
    MEKF_ALGORITHM = "MEKF Algorithm"
    MAGNETORQUER = "Magnetorquer"
    MAX_ALIGN_POINTING_MODE = "Max Align Pointing Mode"
    MODEM = "Modem"
    OPERATIONAL_MODE = "Operational Mode"
    OPTICAL_ATTITUDE_SENSOR = "Optical Attitude Sensor"
    ORBIT = "Orbit"
    PID_ALGORITHM = "PID Algorithm"
    PASSIVE_POINTING_MODE = "Passive Pointing Mode"
    PASSIVE_TRANSMIT_INTERFACE = "Passive Transmit Interface"
    POSITION_SENSOR = "Position Sensor"
    POWER_PROCESSOR = "Power Processor"
    REACTION_WHEEL = "Reaction Wheel"
    RECEIVE_INTERFACE = "Receive Interface"
    RECTANGULAR_FIELD_OF_VIEW = "Rectangular Field Of View"
    SATELLITE = "Satellite"
    SLIDING_MODE_ALGORITHM = "Sliding Mode Algorithm"
    SOLAR_ARRAY = "Solar Array"
    SOLAR_CELL = "Solar Cell"
    SOLAR_PANEL = "Solar Panel"
    SPACE_TARGET = "Space Target"
    SPHERICAL_FUEL_TANK = "Spherical Fuel Tank"
    SPHEROCYLINDER_FUEL_TANK = "Spherocylinder Fuel Tank"
    STATIC_THRUST_CONTROL_ALGORITHM = "Static Thrust Control Algorithm"
    SUBSYSTEM = "Subsystem"
    SURFACE = "Surface"
    SURFACE_MATERIAL = "Surface Material"
    TARGET_GROUP = "Target Group"
    TARGET_GROUP_VECTOR = "Target Group Vector"
    TARGET_VECTOR = "Target Vector"
    TEMP_CONTROLLER_STATE = "Temp Controller State"
    THERMAL_INTERFACE = "Thermal Interface"
    THERMAL_INTERFACE_MATERIAL = "Thermal Interface Material"
    THRUSTER = "Thruster"
    TRIAD_ALGORITHM = "Triad Algorithm"
    VECTOR_SENSOR = "Vector Sensor"



from sedaro_base_client.paths.models_branches_branch_id_cdh_data_internal_data_interfaces_.post import ApiForpost


class ModelsBranchesBranchIdCdhDataInternalDataInterfaces(
    ApiForpost,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_thermal_thermal_interface_materials_.post import ApiForpost


class ModelsBranchesBranchIdThermalThermalInterfaceMaterials(
    ApiForpost,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_gnc_pointing_modes_max_secondary_align_block_id.delete import ApiFordelete
from sedaro_base_client.paths.models_branches_branch_id_gnc_pointing_modes_max_secondary_align_block_id.patch import ApiForpatch


class ModelsBranchesBranchIdGncPointingModesMaxSecondaryAlignBlockId(
    ApiFordelete,
    ApiForpatch,
):
    pass



from typing import TYPE_CHECKING, Dict, List, Union

from pydash.strings import snake_case
from sedaro_base_client.api_client import ApiResponse
from sedaro_base_client.paths.models_branches_branch_id.get import \
    SchemaFor200ResponseBodyApplicationJson

from .block_class_client import BlockClassClient
from .block_client import BlockClient
from .settings import BASE_PACKAGE_NAME
from .utils import (get_class_from_module, import_if_exists,
                    parse_block_crud_response,
                    sanitize_and_enforce_id_in_branch)

if TYPE_CHECKING:
    from .sedaro_api_client import SedaroApiClient


class BranchClient:

    def __init__(self, body: SchemaFor200ResponseBodyApplicationJson, client: 'SedaroApiClient'):
        for k, v in body.items():
            setattr(self, k, v)
        self.data_schema: Dict = body['dataSchema']
        self._sedaro_client = client
        self._block_id_to_type_map: Dict[str, str] = body['blockIdToTypeMap']
        '''Dictionary mapping Sedaro Block ids to the class name of the Block'''
        self._block_class_to_block_group_map: Dict[str, str] = {
            **body['blockClassToBlockGroupMap'], 'ConstantLoad': 'Load', 'Surface': 'Surface'
        }
        # TODO: these blocks ^^^ added in manually, b/c they aren't explicitly in the BG's but are valid block class
        # clients; may be able to refactor this later
        '''Dictionary mapping Block class names to the Sedaro Block Group they are in'''
        self._block_group_names: List[str] = body['blockGroupNames']

    def __str__(self):
        return f'BranchClient(id: {self.id}, name: "{self.name}")'

    def __repr__(self):
        return self.__str__()

    def __getattr__(self, block_type: str) -> BlockClassClient:

        # Valid block class client options that don't have an associated api (won't have a create method)
        bcc_options_without_api = {'ConOps'}
        if block_type in bcc_options_without_api:
            return BlockClassClient(block_type, None, self)

        # check if is a valid option for creating a BlockClassClient & get respective api module file
        cant_create_err_msg = f'Unable to create a "BlockClassClient" from string: "{block_type}". Please check the name and try again.'

        # -- check block type
        if block_type not in self._block_class_to_block_group_map:
            raise AttributeError(cant_create_err_msg)

        # -- check module exists
        block_api_module = import_if_exists(
            f'{BASE_PACKAGE_NAME}.apis.tags.{snake_case(block_type)}_api'
        )
        if block_api_module is None:
            raise AttributeError(cant_create_err_msg)

        # instantiate BCC
        block_api_class = get_class_from_module(block_api_module)
        return BlockClassClient(block_type, block_api_class(self._sedaro_client), self)

    def _process_block_crud_response(self, block_crud_response: ApiResponse) -> str:
        """Updates the local `Branch` data according to the CRUD action completed

        Args:
            block_crud_response (ApiResponse): response from a Sedaro Block CRUD action

        Raises:
            NotImplementedError: if the returned CRUD action isn't create, update, or delete

        Returns:
            str: `block_id`
        """
        block_id, block_data, block_group, action, branch_data_incoming, block_id_to_type_map = parse_block_crud_response(
            block_crud_response
        )

        self._block_id_to_type_map = block_id_to_type_map
        self.data = branch_data_incoming

        return block_id

    def get_block_client(self, id: Union[str, int]):
        """Gets a `BlockClient` associated with the Sedaro Block of the given `id`.

        Args:
            id (Union[str, int]): `id` of the desired Sedaro Block

        Raises:
            TypeError: if the `id` is not an integer or an integer string
            KeyError: if no corresponding Block exists in the Branch

        Returns:
            BlockClient: a client to interact with the corresponding Sedaro Block
        """
        id = sanitize_and_enforce_id_in_branch(self, id)

        # first try with block_type, then try with block_group
        block_type = self._block_id_to_type_map[id]
        try:
            b_c_c: BlockClassClient = getattr(self, block_type)
        except AttributeError as e:
            block_group = self._block_class_to_block_group_map[block_type]
            try:
                b_c_c = BlockClassClient = getattr(self, block_group)
            except AttributeError:
                raise e
        return BlockClient(id, b_c_c)



from sedaro_base_client.paths.models_branches_branch_id_power_eps_power_processor_block_id.patch import ApiForpatch


class ModelsBranchesBranchIdPowerEpsPowerProcessorBlockId(
    ApiForpatch,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_gnc_algorithms_thrust_control_static_thrust_control_.post import ApiForpost


class ModelsBranchesBranchIdGncAlgorithmsThrustControlStaticThrustControl(
    ApiForpost,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_system_geometry_surfaces_materials_.post import ApiForpost


class ModelsBranchesBranchIdSystemGeometrySurfacesMaterials(
    ApiForpost,
):
    pass



from sedaro_base_client.paths.models_branches_branch_id_gnc_algorithms_attitude_determination_triad_.post import ApiForpost


class ModelsBranchesBranchIdGncAlgorithmsAttitudeDeterminationTriad(
    ApiForpost,
):
    pass
